Class MapVoteHistory extends Object
	Config(VotingHistory);

// Using short variable names to keep ini file smallest possible.
struct FHistory
{
	var() config string M;
	var() config int S,V;
};
var() config array<FHistory> H;
var() config bool bNewVer;

// Note: this function is in risk of causing runaway loop if many maps in rotation and history.
static final function GetMapHistory( string MapName, out int Plays, out int Sequence )
{
	local int i;

	// Maplist was generated in old version of this mod, fix it up.
	if( !Default.bNewVer )
	{
		if( Array_Size(Default.H)>0 )
			FixMaplist();
		Default.bNewVer = true;
	}
	MapName = Caps(MapName);
	i = FindIndex(MapName);
	if( i>=0 )
	{
		Sequence = Default.H[i].S;
		Plays = Default.H[i].V;
	}
	else
	{
		AddMap(MapName);
		Sequence = 0;
		Plays = 0;
	}
}
static final function UpdateHistory( string WinMap )
{
	local int i,c;

	c = Array_Size(Default.H);
	for( i=0; i<c; ++i )
	{
		if( Default.H[i].M~=WinMap )
		{
			Default.H[i].S = 0;
			++Default.H[i].V;
		}
		else ++Default.H[i].S;
	}
	StaticSaveConfig();
}

static final function int FindIndex( string MapName )
{
	local int a,b,i;

	// speedy way to find the map if it alread exists
	// a               7                           b
	// 12345678901234568901234567890123456789012345
	// |----------|----------|----------|----------|
	// 1                     <                       too high
	// 2---------------------b                       b = ((b - a)/2) + a
	// 3          >                                  too low
	// 4          a----------b                       a = ((b - a)/2) + a
	// 7               <                             too high
	// 8          a----b                             b = ((b - a)/2) + a
	// 9            >                                too low
	// 10           a--b                             a = ((b - a)/2) + a
	// 11            >                               too low
	// 12            a-b
	// 13             >                              too low
	// 14             ab
	// 15             >                              too low
	// 16             b                              a==b

	if( Array_Size(Default.H)==0 )
		return -1;

	a = 1;
	b = Array_Size(Default.H);

	while(true)
	{
		i = ((b-a)/2)+a;
		if( Default.H[i-1].M==MapName )  // check for a match
			return (i-1); // found

		if( a==b ) // Not found
			return -1;

		if( Default.H[i-1].M>MapName )  //check mid-way
			b = i;    // too high
		else
		{
			if(a == i)
				a = b;
			else
				a = i;    // too low
		}
	}
}
static final function int AddMap( string MapName )
{
	local int x,n;

	if( MapName=="" )
		return -1;

	n = Array_Size(Default.H);
	if( n==0 )  // brand new list
	{
		Default.H[0].M = MapName;
		Default.H[0].S = 0;
		Default.H[0].V = 0;
		return 0;
	}

	// search list for map
	for(x=0; x<n; x++)
	{
		if( Default.H[x].M>MapName )  // Map should be inserted here
		{
			Array_Insert(Default.H,x);
			Default.H[x].M = MapName;
			Default.H[x].S = 0;
			Default.H[x].V = 0;
			return x;
		}
	}

	// didnt find insertion point so add at end
	Default.H[n].M = MapName;
	Default.H[n].S = 0;
	Default.H[n].V = 0;
	return n;
}
static final function FixMaplist()
{
	local array<FHistory> TH;
	local int i,n,j;
	
	n = Array_Size(Default.H);
	TH = Default.H;
	Array_Size(Default.H,0);
	for( i=0; i<n; ++i )
	{
		j = AddMap(Caps(TH[i].M));
		Default.H[j].S = TH[i].S;
		Default.H[j].V = TH[i].V;
	}
}

defaultproperties
{
}
